'///***************************************************************************
'// File:        GENERIC.C
'//
'// Purpose:     Template for creating XLLs for Microsoft Excel.
'//
'//              This file contains sample code you can use as 
'//              a template for writing your own Microsoft Excel XLLs. 
'//              An XLL is a DLL that stands alone, that is, you
'//              can open it by choosing the Open command from the
'//              File menu. This code demonstrates many of the features 
'//              of the Microsoft Excel C API.

'//              
'//              GENERIC.XLL can also be added with the
'//              Add-in Manager.
'//              

#pragma once
#include once "windows.bi"
#include once "xlcall.bi"
#include once "framewrkc.bi"
'#inclib "xlcall13"   'xlcall.cpp in a static library ,remember to omit lib when  using inclib even if file starts with lib
#include once "xlcall32_cpp.bas" 'callback and excel12proc entry point
#include once "XL_Pascalstr.bi"  'routines for reading and writing zstrings or wstrings
#include once "FB_Operarr.bas"
#include once  "FB_XLRegfunc.bas" 'function layouts and parameter definitions
#include once  "Myfirstfunctions.bas"   'functions in this add-in

dim shared xlAddinName as wstring*255 = "MyFirstXll"

extern     g_hWndMain as HWND
dim shared g_hWndMain as HWND = NULL
extern     g_hInst as HANDLE
dim shared g_hInst as HANDLE = NULL

extern g_InitAlready AS short
dim shared g_InitAlready AS short  

'///***************************************************************************
'// DllMain()
'//
'// Purpose:
'//
'//      Windows calls DllMain, for both initialization and termination.
'//		It also makes calls on both a per-process and per-thread basis,
'//		so several initialization calls can be made if a process is multithreaded.
'//
'//      This function is called when the DLL is first loaded, with a dwReason
'//      of DLL_PROCESS_ATTACH.
'//
'// Parameters:
'//
'//      HANDLE hDLL         Module handle.
'//      DWORD dwReason,     Reason for call
'//      LPVOID lpReserved   Reserved
'//
'// Returns: 
'//      The function returns TRUE (1) to indicate success. If, during
'//      per-process initialization, the function returns zero, 
'//      the system cancels the process.
'//
'// Comments:
'//
'// History:  Date       Author        Reason
'///***************************************************************************

Public Function DllMain stdcall  Alias "XLLMAIN" _
    ( _
        Byval hDLL As HINSTANCE, _
        Byval fdwReason As DWORD, _
        Byval lpvReserved As LPVOID _
    ) As BOOL

   'check if global array with parameters for register function is dimensined already
   IF g_InitAlready  = 0 THEN
      
         InitFuncParameters() 'initialize the table with register parameters
      ELSE
      g_InitAlready  = 0
   END IF 
                  
    Select Case (fdwReason)
       Case DLL_PROCESS_ATTACH
         
'		// The instance handle passed into DllMain is saved
'		// in the global variable g_hInst for later use.

		g_hInst = hDLL
		
    Case DLL_PROCESS_DETACH

    Case DLL_THREAD_ATTACH

    Case DLL_THREAD_DETACH

    End Select

    Return 1
End Function

'///***************************************************************************
'// xlAutoOpen()
'//
'// Purpose: 
'//      Microsoft Excel call this function when the DLL is loaded.
'//
'//      Microsoft Excel uses xlAutoOpen to load XLL files.
'//      When you open an XLL file, the only action
'//      Microsoft Excel takes is to call the xlAutoOpen function.
'//
'//      More specifically, xlAutoOpen is called:
'//
'//       - when you open this XLL file from the File menu,
'//       - when this XLL is in the XLSTART directory, and is
'//         automatically opened when Microsoft Excel starts,
'//       - when Microsoft Excel opens this XLL for any other reason, or
'//       - when a macro calls REGISTER(), with only one argument, which is the
'//         name of this XLL.
'//
'//      xlAutoOpen is also called by the Add-in Manager when you add this XLL 
'//      as an add-in. The Add-in Manager first calls xlAutoAdd, then calls
'//      REGISTER("EXAMPLE.XLL"), which in turn calls xlAutoOpen.
'//
'//      xlAutoOpen should:
'//
'//       - register all the functions you want to make available while this
'//         XLL is open,
'//
'//       - add any menus or menu items that this XLL supports,
'//
'//       - perform any other initialization you need, and
'//
'//       - return 1 if successful, or return 0 if your XLL cannot be opened.
'//
'// Parameters:
'//
'// Returns: 
'//
'//      int         1 on success, 0 on failure
'//
'// Comments:
'//
'// History:  Date       Author        Reason
'///***************************************************************************

'__declspec(dllexport) int WINAPI xlAutoOpen(void)
extern "Windows-MS"
function xlAutoOpen stdcall alias "xlAutoOpen"()  as LONG export
   
	static xDLL as XLOPER12  ' name of this DLL //	
	dim as long i, iret, j '			   // Loop indices //
	STATIC  ExTernDll AS  XLOPER12 'Handle for the Name of the DLL in g_szDLLName   

'	//
'	// In the following block of code the name of the XLL is obtained by
'	// calling xlGetName. This name is used as the first argument to the
'	// REGISTER function to specify the name of the XLL. Next, the XLL loops
'	// through the g_rgWorksheetFuncs[] table, and the g_rgCommandFuncs[]
'	// tableregistering each function in the table using xlfRegister. 
'	// Functions must be registered before you can add a menu item.
'	//
	
	 DIM szFile1 AS WSTRING*260

    'g_hInst  is handle of DLL/XLL file. Obtained through Dllmain
     'get filename and path of this XLL file
     GetModuleFileNameW g_hInst, szFile1, SIZEOF (szFile1)
      
          
   iRet = Excel12 (xlGetName OR xlIntl , BYVAL @xDLL, 0 ) 
       
    'the array g_rgWorksheetFuncs() already dimensioned and initialized in DLL entry function XLLmain
       
	FOR i = 0 TO g_rgWorksheetFuncsRows - 1

     iret = Excel12 (xlfRegister OR xlIntl ,  0, 1 + g_rgWorksheetFuncsCols,_
         BYVAL @xDLL, _
         BYVAL Tempstr12(g_rgWorksheetFuncs(i, 0)), _
         BYVAL TempStr12(g_rgWorksheetFuncs(i, 1)), _
         BYVAL TempStr12(g_rgWorksheetFuncs(i, 2)), _
         BYVAL TempStr12(g_rgWorksheetFuncs(i, 3)), _
         BYVAL TempStr12(g_rgWorksheetFuncs(i, 4)), _
         BYVAL TempStr12(g_rgWorksheetFuncs(i, 5)), _
         BYVAL TempStr12(g_rgWorksheetFuncs(i, 6)), _
         BYVAL TempStr12(g_rgWorksheetFuncs(i, 7)), _
         BYVAL TempStr12(g_rgWorksheetFuncs(i, 8))  )
     
   FreeAllTempMemory()
   NEXT                 
	
	'// Free the XLL filename //
   Excel12 xlFree OR xlIntl ,  0, 1, BYVAL @xDLL  
	return 1
end function


'///***************************************************************************
'// xlAutoClose()
'//
'// Purpose: Microsoft Excel call this function when the DLL is unloaded.
'//
'//      xlAutoClose is called by Microsoft Excel:
'//
'//       - when you quit Microsoft Excel, or 
'//       - when a macro sheet calls UNREGISTER(), giving a string argument
'//         which is the name of this XLL.
'//
'//      xlAutoClose is called by the Add-in Manager when you remove this XLL from
'//      the list of loaded add-ins. The Add-in Manager first calls xlAutoRemove,
'//      then calls UNREGISTER("GENERIC.XLL"), which in turn calls xlAutoClose.
'// 
'//      xlAutoClose is called by GENERIC.XLL by the function fExit. This function
'//      is called when you exit Generic.
'// 
'//      xlAutoClose should:
'// 
'//       - Remove any menus or menu items that were added in xlAutoOpen,
'// 
'//       - do any necessary global cleanup, and
'// 
'//       - delete any names that were added (names of exported functions, and 
'//         so on). Remember that registering functions may cause names to 
'//         be created.
'// 
'//      xlAutoClose does NOT have to unregister the functions that were registered
'//      in xlAutoOpen. This is done automatically by Microsoft Excel after
'//      xlAutoClose returns.
'// 
'//      xlAutoClose should return 1.
'//
'// Parameters:
'//
'// Returns: 
'//
'//      int         1
'//
'// Comments:
'//
'// History:  Date       Author        Reason
'///***************************************************************************

'__declspec(dllexport) int WINAPI xlAutoClose(void)
function xlAutoClose stdcall alias "xlAutoClose"() as long export
dim i as Long
dim xRes as 	XLOPER12 

'	//
'	// This block first deletes all names added by xlAutoOpen or
'	// xlAutoRegister12. Next, it checks if the drop-down menu Generic still
'	// exists. If it does, it is deleted using xlfDeleteMenu. It then checks
'	// if the Test toolbar still exists. If it is, xlfDeleteToolbar is
'	// used to delete it.
'	//

'	//
'	// Due to a bug in Excel the following code to delete the defined names
'	// does not work.  There is no way to delete these
'	// names once they are Registered
'	// The code is left in, in hopes that it will be
'	// fixed in a future version.


	FOR i = 0 TO  g_rgWorksheetFuncsRows - 1
      Excel12 xlfSetName OR xlIntl ,  0, 1, BYVAL TempStr12(g_rgWorksheetFuncs(i, 2))
      FreeAllTempMemory()
   NEXT

	return 1
end function





'///***************************************************************************
'// xlAutoAdd()
'//
'// Purpose:
'//
'//      This function is called by the Add-in Manager only. When you add a
'//      DLL to the list of active add-ins, the Add-in Manager calls xlAutoAdd()
'//      and then opens the XLL, which in turn calls xlAutoOpen.
'//
'// Parameters:
'//
'// Returns: 
'//
'//      int         1
'//
'// Comments:
'//
'// History:  Date       Author        Reason
'///***************************************************************************

'__declspec(dllexport) int WINAPI xlAutoAdd(void)
function xlAutoAdd stdcall alias "xlAutoAdd"() as long export
   
	dim szBuf as wstring*255
   szBuf = "My first XLL add-in!"
    dim gg as xloper12
    gg.xltype= xlstring
    gg.val.str=WritePascalStrW(szBuf)
    
    
	'// Display a dialog box indicating that the XLL was successfully added //
	'Excel12 xlcAlert, 0, 2, byval TempStr12(szBuf),byval  TempInt12(2)
   Excel12 xlcAlert, 0, 2, byval @gg,byval  TempInt12(2)
	return 1
END FUNCTION

'///***************************************************************************
'// xlAutoRemove()
'//
'// Purpose:
'//
'//      This function is called by the Add-in Manager only. When you remove
'//      an XLL from the list of active add-ins, the Add-in Manager calls
'//      xlAutoRemove() and then UNREGISTER("GENERIC.XLL").
'//   
'//      You can use this function to perform any special tasks that need to be
'//      performed when you remove the XLL from the Add-in Manager's list
'//      of active add-ins. For example, you may want to delete an
'//      initialization file when the XLL is removed from the list.
'//
'// Parameters:
'//
'// Returns: 
'//
'//      int         1
'//
'// Comments:
'//
'// History:  Date       Author        Reason
'///***************************************************************************

'__declspec(dllexport) int WINAPI xlAutoRemove(void)
function xlAutoRemove stdcall alias "xlAutoRemove"() as long export
   
	'// Show a dialog box indicating that the XLL was successfully removed //
	Excel12 xlcAlert, 0, 2, TempStr12("Thank you for removing GENERIC.XLL!"), TempInt12(2)
	return 1
END FUNCTION

'///***************************************************************************
'// xlAddInManagerInfo12()
'//
'// Purpose:
'//
'//      This function is called by the Add-in Manager to find the long name
'//      of the add-in. If xAction = 1, this function should return a string
'//      containing the long name of this XLL, which the Add-in Manager will use
'//      to describe this XLL. If xAction = 2 or 3, this function should return
'//      #VALUE!.
'//
'// Parameters:
'//
'//      LPXLOPER12 xAction  What information you want. One of:
'//                            1 = the long name of the
'//                                add-in
'//                            2 = reserved
'//                            3 = reserved
'//
'// Returns: 
'//
'//      LPXLOPER12          The long name or #VALUE!.
'//
'// Comments:
'//
'// History:  Date       Author        Reason
'///***************************************************************************

'__declspec(dllexport) LPXLOPER12 WINAPI xlAddInManagerInfo12()
function xlAddInManagerInfo12 stdcall alias "xlAddInManagerInfo12"(xAction as LPXLOPER12) as LPXLOPER12 export
   

	static as XLOPER12 xInfo, xIntAction

'	//
'	// This code coerces the passed-in value to an integer. This is how the
'	// code determines what is being requested. If it receives a 1, 
'	// it returns a string representing the long name. If it receives 
'	// anything else, it returns a #VALUE! error.
'	//
     dim intAddinname AS WSTRING*500
     intAddinname = xlAddinName
	Excel12(xlCoerce, @xIntAction, 2, xAction, TempInt12(xltypeInt))

	if xIntAction.val.w = 1 then
	
		xInfo.xltype = xltypeStr
		xInfo.val.str = WritepascalStrW("Generic Standalone DLL")
	
	else
	
		xInfo.xltype = xltypeErr
		xInfo.val.err = xlerrValue
	end if

'	//Word of caution - returning static XLOPERs/XLOPER12s is not thread safe
'	//for UDFs declared as thread safe, use alternate memory allocation mechanisms
	return @xInfo
END FUNCTION


end extern